<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
				xmlns:x="urn:xslt:alias"
                xmlns:xlink="http://www.w3.org/1999/xlink"
                xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
                xmlns:mml="http://www.w3.org/1998/Math/MathML"
				xmlns:my="urn:my:functions"
                version="2.0">
   
   <xsl:output method="text" />
   <xsl:namespace-alias stylesheet-prefix="x" result-prefix="xsl"/>   
   <xsl:variable name="crlf" select="'&#10;'" />
   <xsl:strip-space elements="*"/>
   
   <!-- =========================================================================================== -->
   <!-- Write a template that converts every element to a declaration -->
   <!-- ******************************************************************************************* -->   
   <xsl:template match="/">                 
         <xsl:apply-templates select="/declarations/elements/element">
			<xsl:sort select="@name"/>
         </xsl:apply-templates>               
   </xsl:template>
   
   <!-- ================================================================================================ -->
   <!-- Create template for each element -->
   <!-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ -->
   <xsl:template match="element">
   
      <xsl:variable name="curr-element" select="@name"/>
      <xsl:variable name="attributes" select="/declarations/attributes/attribute/attributeDeclaration[@element eq $curr-element]"/>
	
      <xsl:variable name="min-name">
         <xsl:call-template name="base26">
		    <xsl:with-param name="number" select="@dtdOrder"/>
		 </xsl:call-template>
      </xsl:variable>	
	  
	  <xsl:value-of select="$crlf"/>
	  <xsl:text>&lt;!ELEMENT </xsl:text>
	  <xsl:value-of select="$min-name"/>
	  <xsl:text> </xsl:text>
	  <xsl:call-template name="change-model">
	     <xsl:with-param name="str" select="content-model/@minified"/>
	  </xsl:call-template>
	  <xsl:text>&gt;</xsl:text>
      <xsl:value-of select="$crlf"/>	

      <xsl:for-each select="$attributes">
	     <xsl:text>&lt;!ATTLIST </xsl:text>
		 <xsl:value-of select="$min-name"/>
		 <xsl:text> </xsl:text>	  
		 <xsl:value-of select="parent::attribute/@name"/>
		 <xsl:text> </xsl:text>
		 <xsl:value-of select="@type"/>
		 <xsl:text> </xsl:text>
		 <xsl:value-of select="@mode"/>
		 
		 <xsl:if test="@defaultValue">
		    <xsl:text> "</xsl:text>
			<xsl:value-of select="@defaultValue"/>
			<xsl:text>"</xsl:text>
		 </xsl:if>
		 
		 <xsl:text>&gt;</xsl:text>
		 <xsl:value-of select="$crlf"/>
      </xsl:for-each>	  
   </xsl:template>   
   
   <!-- =========================================================================================== -->
   <!-- Assumes numbers are from 0 up. Convert to a letter-only base 26 representation. -->
   <!-- ******************************************************************************************* -->
   <xsl:template name="base26">
      <xsl:param name="number" select="0"/>
	  <xsl:param name="accumulator" select="''"/>
	  <xsl:variable name="remainder" select="number($number) mod 26"/>
	  <xsl:variable name="quotient" select="floor((number($number) - $remainder) div 26)"/>
	  
	  <xsl:choose>
	     <xsl:when test="number($number) eq 0">
		    <xsl:text>a</xsl:text>
		 </xsl:when>
		 
		 <xsl:when test="$quotient le 0">
		    <xsl:value-of select="my:number-to-letter($remainder)"/>
		    <xsl:value-of select="$accumulator"/>
		 </xsl:when>
		 
		 <xsl:otherwise>
		     <xsl:call-template name="base26">
			    <xsl:with-param name="number" select="$quotient"/>
				<xsl:with-param name="accumulator" select="concat( my:number-to-letter($remainder), $accumulator )"/>				
			 </xsl:call-template>
		 </xsl:otherwise>
	  </xsl:choose>
   </xsl:template>
   
   <!-- =========================================================================================== -->
   <!-- Assumes numbers are from 0 to 25 -->
   <!-- ******************************************************************************************* -->
   <xsl:function name="my:number-to-letter">
       <xsl:param name="number"/>
	   <xsl:variable name="alpha" select="'abcdefghijklmnopqrstuvwxyz'"/>
	   <xsl:value-of select="substring($alpha, number($number) + 1, 1)"/>
   </xsl:function>
   

   <!-- =========================================================== -->
   <!-- TEMPLATE: change-model -->
   <!-- =========================================================== -->
   <xsl:template name="change-model">
      <xsl:param name="str"/>
      
      <xsl:choose>
         <!-- Base case: no more characters -->
         <xsl:when test="not($str)"/>
         
         <!-- Starts with a reserved string -->         
         <xsl:when test="normalize-space($str)='EMPTY'">
            <xsl:text>EMPTY</xsl:text>
         </xsl:when>

         <!-- Starts with a reserved string -->         
         <xsl:when test="normalize-space($str) = 'ANY'">
            <xsl:text>ANY</xsl:text>
         </xsl:when>
         
         <xsl:when test="starts-with($str, '#PCDATA')">
            <xsl:text>#PCDATA</xsl:text>
            <xsl:call-template name="change-model">
               <xsl:with-param name="str" select="substring-after($str, '#PCDATA')"/>
            </xsl:call-template>
         </xsl:when>
                  
         <xsl:when test="starts-with($str, '(')
                         or starts-with($str, ')')
                         or starts-with($str, '+')
                         or starts-with($str, '|')
                         or starts-with($str, '?')
                         or starts-with($str, '*')
                         or starts-with($str, ',')">
            <xsl:value-of select="substring($str, 1, 1)"/>
            <xsl:call-template name="change-model">
               <xsl:with-param name="str" select="substring($str, 2)"/>
            </xsl:call-template>
         </xsl:when>
         
         <!-- Must be the start of an element name - must retrieve it -->
         <xsl:otherwise>
            <xsl:variable name="element-name">
               <xsl:call-template name="find-element-name">
                  <xsl:with-param name="str" select="$str"/>
               </xsl:call-template>
            </xsl:variable>
       
	        <!-- Find element and its dtd order; then convert to new name -->
			<xsl:variable name="order" select="/declarations/elements/element[@name eq $element-name]/@dtdOrder"/>
            <xsl:call-template name="base26">
			   <xsl:with-param name="number" select="$order"/>
			</xsl:call-template>
       
            <!-- Now continue recursing -->
            <xsl:call-template name="change-model">
               <xsl:with-param name="str" select="substring-after($str, normalize-space($element-name))"/>
            </xsl:call-template>
         </xsl:otherwise>
      </xsl:choose>
   </xsl:template>   

   <!-- =========================================================== -->
   <!-- TEMPLATE: find-element-name -->
   <!-- =========================================================== -->
   <xsl:template name="find-element-name">
      <xsl:param name="str"/>
         
         <xsl:choose>
            <!-- Cover your ass base case (impossible) -->
            <xsl:when test="not($str)"/>
            
            <!-- Real base case: found a delimiter, so stop -->
            <xsl:when test="starts-with($str, ')')
                         or starts-with($str, ' ')
                         or starts-with($str, '+')
                         or starts-with($str, '|')
                         or starts-with($str, '?')
                         or starts-with($str, '*')
                         or starts-with($str, ',')"/>
               
            <!-- Otherwise, still in element name -->
            <xsl:otherwise>
               <xsl:value-of select="substring($str, 1, 1)"/>
               <xsl:call-template name="find-element-name">
                  <xsl:with-param name="str" select="substring($str, 2)"/>
               </xsl:call-template>
            </xsl:otherwise>
         </xsl:choose>
   </xsl:template>
   
</xsl:stylesheet>